@@ -710,6 +710,7 @@ class ControlFlowGraph(_bytecode.BaseBytecode):
         del block[index:]
 
         block2 = BasicBlock(instructions)
+        block2.next_block = block.next_block
         block.next_block = block2
 
         for block in self[block_index + 1 :]:
@@ -731,12 +732,18 @@ class ControlFlowGraph(_bytecode.BaseBytecode):
             if id(block) in seen_block_ids:
                 continue
             seen_block_ids.add(id(block))
+            fall_through = True
             for i in block:
-                if isinstance(i, Instr) and isinstance(i.arg, BasicBlock):
-                    stack.append(i.arg)
+                if isinstance(i, Instr):
+                    if isinstance(i.arg, BasicBlock):
+                        stack.append(i.arg)
+                    if i.is_final():
+                        fall_through = False
                 elif isinstance(i, TryBegin):
                     assert isinstance(i.target, BasicBlock)
                     stack.append(i.target)
+            if fall_through and block.next_block:
+                stack.append(block.next_block)
 
         return [b for b in self if id(b) not in seen_block_ids]
 
@@ -865,9 +872,7 @@ class ControlFlowGraph(_bytecode.BaseBytecode):
                     #   the new one since the blocks are disconnected.
                     if last_instr.is_final() and temp:
                         old_block.append(TryEnd(try_begins[active_try_begin][-1]))
-                        new_tb = TryBegin(
-                            active_try_begin.target, active_try_begin.push_lasti
-                        )
+                        new_tb = active_try_begin.copy()
                         block.append(new_tb)
                         # Add this new TryBegin to the map to properly update
                         # the target.
