@@ -26,18 +26,25 @@ import bytecode as _bytecode
 from bytecode.flags import CompilerFlags
 from bytecode.instr import (
     _UNSET,
+    BINARY_OPS,
     BITFLAG2_OPCODES,
     BITFLAG_OPCODES,
+    COMMON_CONSTANT_OPS,
     DUAL_ARG_OPCODES,
     DUAL_ARG_OPCODES_SINGLE_OPS,
+    FORMAT_VALUE_OPS,
     INTRINSIC,
     INTRINSIC_1OP,
     INTRINSIC_2OP,
     PLACEHOLDER_LABEL,
+    SPECIAL_OPS,
     UNSET,
     BaseInstr,
+    BinaryOp,
     CellVar,
+    CommonConstant,
     Compare,
+    FormatValue,
     FreeVar,
     Instr,
     InstrArg,
@@ -46,6 +53,7 @@ from bytecode.instr import (
     Intrinsic2Op,
     Label,
     SetLineno,
+    SpecialMethod,
     TryBegin,
     TryEnd,
     _check_arg_int,
@@ -1056,7 +1064,10 @@ class ConcreteBytecode(_bytecode._BaseBytecodeList[Union[ConcreteInstr, SetLinen
                         arg = locals_lookup[c_arg]
                 elif opcode in _opcode.hasname:
                     if opcode in BITFLAG_OPCODES:
-                        arg = (bool(c_arg & 1), self.names[c_arg >> 1])
+                        arg = (
+                            bool(c_arg & 1),
+                            self.names[c_arg >> 1],
+                        )
                     elif opcode in BITFLAG2_OPCODES:
                         arg = (bool(c_arg & 1), bool(c_arg & 2), self.names[c_arg >> 2])
                     else:
@@ -1082,6 +1093,20 @@ class ConcreteBytecode(_bytecode._BaseBytecodeList[Union[ConcreteInstr, SetLinen
                     arg = Intrinsic1Op(c_arg)
                 elif opcode in INTRINSIC_2OP:
                     arg = Intrinsic2Op(c_arg)
+                elif opcode in BINARY_OPS:
+                    arg = BinaryOp(c_arg)
+                elif opcode in COMMON_CONSTANT_OPS:
+                    arg = CommonConstant(c_arg)
+                elif opcode in SPECIAL_OPS:
+                    arg = SpecialMethod(c_arg)
+                elif opcode in FORMAT_VALUE_OPS:
+                    if opcode in BITFLAG_OPCODES:
+                        arg = (
+                            bool(c_arg & 1),
+                            FormatValue(c_arg >> 1),
+                        )
+                    else:
+                        arg = FormatValue(c_arg)
                 else:
                     arg = c_arg
 
@@ -1143,7 +1168,7 @@ class ConcreteBytecode(_bytecode._BaseBytecodeList[Union[ConcreteInstr, SetLinen
 
 
 class _ConvertBytecodeToConcrete:
-    # XXX document attributes
+    # FIXME document attributes
 
     #: Default number of passes of compute_jumps() before giving up.  Refer to
     #: assemble_jump_offsets() in compile.c for background.
@@ -1316,9 +1341,13 @@ class _ConvertBytecodeToConcrete:
                         isinstance(arg, tuple)
                         and len(arg) == 2
                         and isinstance(arg[0], bool)
-                        and isinstance(arg[1], str)
                     ), arg
-                    index = self.add(self.names, arg[1])
+                    if isinstance(arg[1], str):
+                        index = self.add(self.names, arg[1])
+                    elif isinstance(arg, FormatValue):
+                        index = int(arg)
+                    else:
+                        assert False, arg  # noqa
                     c_arg = int(arg[0]) + (index << 1)
                 elif opcode in BITFLAG2_OPCODES:
                     assert (
