@@ -13,7 +13,7 @@ except ImportError:
     from typing_extensions import TypeGuard  # type: ignore
 
 import bytecode as _bytecode
-from bytecode.utils import PY311, PY312, PY313
+from bytecode.utils import PY311, PY312, PY313, PY314
 
 # --- Instruction argument tools and
 
@@ -22,20 +22,48 @@ MIN_INSTRUMENTED_OPCODE = getattr(_opcode, "MIN_INSTRUMENTED_OPCODE", 256)
 # Instructions relying on a bit to modify its behavior.
 # The lowest bit is used to encode custom behavior.
 BITFLAG_OPCODES = (
-    (_opcode.opmap["LOAD_GLOBAL"], _opcode.opmap["LOAD_ATTR"])
-    if PY312
-    else (_opcode.opmap["LOAD_GLOBAL"],)
-    if PY311
-    else ()
+    (
+        _opcode.opmap["BUILD_INTERPOLATION"],
+        _opcode.opmap["LOAD_GLOBAL"],
+        _opcode.opmap["LOAD_ATTR"],
+    )
+    if PY314
+    else (
+        (_opcode.opmap["LOAD_GLOBAL"], _opcode.opmap["LOAD_ATTR"])
+        if PY312
+        else ((_opcode.opmap["LOAD_GLOBAL"],) if PY311 else ())
+    )
 )
 
 BITFLAG2_OPCODES = (_opcode.opmap["LOAD_SUPER_ATTR"],) if PY312 else ()
 
+# Binary op opcode which has a dedicated arg
+BINARY_OPS = (_opcode.opmap["BINARY_OP"],) if PY311 else ()
+
 # Intrinsic related opcodes
 INTRINSIC_1OP = (_opcode.opmap["CALL_INTRINSIC_1"],) if PY312 else ()
 INTRINSIC_2OP = (_opcode.opmap["CALL_INTRINSIC_2"],) if PY312 else ()
 INTRINSIC = INTRINSIC_1OP + INTRINSIC_2OP
 
+# Small integer related opcode
+SMALL_INT_OPS = (_opcode.opmap["LOAD_SMALL_INT"],) if PY314 else ()
+
+# Special method loading related opcodes
+SPECIAL_OPS = (_opcode.opmap["LOAD_SPECIAL"],) if PY314 else ()
+
+# Common constant loading related opcodes
+COMMON_CONSTANT_OPS = (_opcode.opmap["LOAD_COMMON_CONSTANT"],) if PY314 else ()
+
+# Value formatting related opcodes (only handle CONVERT_VALUE and BUILD_INTERPOLATION)
+FORMAT_VALUE_OPS = (
+    (
+        _opcode.opmap["CONVERT_VALUE"],
+        _opcode.opmap["BUILD_INTERPOLATION"],
+    )
+    if PY314
+    else ((_opcode.opmap["CONVERT_VALUE"],) if PY313 else ())
+)
+
 HASJABS = () if PY313 else _opcode.hasjabs
 if sys.version_info >= (3, 13):
     HASJREL = _opcode.hasjump
@@ -51,6 +79,11 @@ if PY313:
         _opcode.opmap["STORE_FAST_LOAD_FAST"],
         _opcode.opmap["STORE_FAST_STORE_FAST"],
     )
+    if PY314:
+        DUAL_ARG_OPCODES = (
+            *DUAL_ARG_OPCODES,
+            _opcode.opmap["LOAD_FAST_BORROW_LOAD_FAST_BORROW"],
+        )
     DUAL_ARG_OPCODES_SINGLE_OPS = {
         _opcode.opmap["LOAD_FAST_LOAD_FAST"]: ("LOAD_FAST", "LOAD_FAST"),
         _opcode.opmap["STORE_FAST_LOAD_FAST"]: ("STORE_FAST", "LOAD_FAST"),
@@ -129,6 +162,8 @@ class BinaryOp(enum.IntEnum):
     INPLACE_SUBTRACT = 23
     INPLACE_TRUE_DIVIDE = 24
     INPLACE_XOR = 25
+    if PY314:
+        SUBSCR = 26
 
 
 @enum.unique
@@ -156,6 +191,34 @@ class Intrinsic2Op(enum.IntEnum):
     INTRINSIC_SET_FUNCTION_TYPE_PARAMS = 4
 
 
+@enum.unique
+class FormatValue(enum.IntEnum):
+    STR = 1
+    REPR = 2
+    ASCII = 3
+
+
+@enum.unique
+class SpecialMethod(enum.IntEnum):
+    """Special method names used with LOAD_SPECIAL"""
+
+    ENTER = 0
+    EXIT = 1
+    AENTER = 2
+    AEXIT = 3
+
+
+@enum.unique
+class CommonConstant(enum.IntEnum):
+    """Common constants names used with LOAD_COMMON_CONSTANT"""
+
+    ASSERTION_ERROR = 0
+    NOT_IMPLEMENTED_ERROR = 1
+    BUILTIN_TUPLE = 2
+    BUILTIN_ALL = 3
+    BUILTIN_ANY = 4
+
+
 # This make type checking happy but means it won't catch attempt to manipulate an unset
 # statically. We would need guard on object attribute narrowed down through methods
 class _UNSET(int):
@@ -357,6 +420,7 @@ STATIC_STACK_EFFECTS: Dict[str, Tuple[int, int]] = {
     "FORMAT_SIMPLE": (-1, 1),  # new in 3.13
     "FORMAT_SPEC": (-2, 1),  # new in 3.13
     "TO_BOOL": (-1, 1),  # new in 3.13
+    "BUILD_TEMPLATE": (-2, 1),  # new in 3.14
 }
 
 
@@ -385,6 +449,7 @@ DYNAMIC_STACK_EFFECTS: Dict[
     "FORMAT_VALUE": lambda effect, arg, jump: (effect - 1, 1),
     # FOR_ITER needs TOS to be an iterator, hence a prerequisite of 1 on the stack
     "FOR_ITER": lambda effect, arg, jump: (effect, 0) if jump else (-1, 2),
+    "BUILD_INTERPOLATION": lambda effect, arg, jump: (-(2 + (arg & 1)), 1),
     **{
         # Instr(UNPACK_* , n) pops 1 and pushes n
         k: lambda effect, arg, jump: (-1, effect + 1)
@@ -660,6 +725,7 @@ class BaseInstr(Generic[A]):
             arg = None
         # 3.11 where LOAD_GLOBAL arg encode whether or we push a null
         # 3.12 does the same for LOAD_ATTR
+        # 3.14 does this for BUILD_INTERPOLATION
         elif self._opcode in BITFLAG_OPCODES and isinstance(self._arg, tuple):
             assert len(self._arg) == 2
             arg = self._arg[0]
@@ -802,8 +868,15 @@ InstrArg = Union[
     FreeVar,
     "_bytecode.BasicBlock",
     Compare,
+    FormatValue,
+    BinaryOp,
+    Intrinsic1Op,
+    Intrinsic2Op,
+    CommonConstant,
+    SpecialMethod,
     Tuple[bool, str],
     Tuple[bool, bool, str],
+    Tuple[bool, FormatValue],
     Tuple[Union[str, CellVar, FreeVar], Union[str, CellVar, FreeVar]],
 ]
 
@@ -817,7 +890,7 @@ class Instr(BaseInstr[InstrArg]):
             arg = const_key(arg)
         return (self._location, self._name, arg)
 
-    def _check_arg(self, name: str, opcode: int, arg: InstrArg) -> None:
+    def _check_arg(self, name: str, opcode: int, arg: InstrArg) -> None:  # noqa: C901
         if name == "EXTENDED_ARG":
             raise ValueError(
                 "only concrete instruction can contain EXTENDED_ARG, "
@@ -854,7 +927,7 @@ class Instr(BaseInstr[InstrArg]):
                     and isinstance(arg[1], str)
                 ):
                     raise TypeError(
-                        "operation %s argument must be a tuple[bool, str], "
+                        "operation %s argument must be a tuple[bool, str | FormatValue], "
                         "got %s (value=%s)" % (name, type(arg).__name__, str(arg))
                     )
 
@@ -910,6 +983,37 @@ class Instr(BaseInstr[InstrArg]):
                     "Compare, got %s" % (name, type(arg).__name__)
                 )
 
+        elif opcode in BINARY_OPS:
+            if not isinstance(arg, BinaryOp):
+                if isinstance(arg, int):
+                    try:
+                        arg = BinaryOp(arg)
+                    except Exception as e:
+                        raise TypeError(
+                            "operation %s argument type must be "
+                            "coercible to BinaryOp, got %s" % (name, type(arg).__name__)
+                        ) from e
+                else:
+                    raise TypeError(
+                        "operation %s argument type must be "
+                        "BinaryOp, got %s" % (name, type(arg).__name__)
+                    )
+
+        # We do not enforce constant immortality since which constants are
+        # immortal may differ between recompilation and execution.
+
+        elif opcode in SMALL_INT_OPS:
+            if not isinstance(arg, int):
+                raise TypeError(
+                    "operation %s argument type must be "
+                    "int, got %s" % (name, type(arg).__name__)
+                )
+            if arg < 0 or arg > 255:
+                raise ValueError(
+                    "operation %s argument type must be an "
+                    "int between 0 and 255, got %s" % (name, arg)
+                )
+
         elif opcode in INTRINSIC_1OP:
             if not isinstance(arg, Intrinsic1Op):
                 raise TypeError(
@@ -924,5 +1028,45 @@ class Instr(BaseInstr[InstrArg]):
                     "Intrinsic2Op, got %s" % (name, type(arg).__name__)
                 )
 
+        elif opcode in SPECIAL_OPS:
+            if not isinstance(arg, SpecialMethod):
+                raise TypeError(
+                    "operation %s argument type must be "
+                    "SpecialMethod, got %s" % (name, type(arg).__name__)
+                )
+        elif opcode in COMMON_CONSTANT_OPS:
+            if not isinstance(arg, CommonConstant):
+                raise TypeError(
+                    "operation %s argument type must be "
+                    "CommonConstant, got %s" % (name, type(arg).__name__)
+                )
+
+        elif opcode in FORMAT_VALUE_OPS:
+            if opcode in BITFLAG_OPCODES:
+                if not (
+                    isinstance(arg, tuple)
+                    and len(arg) == 2
+                    and isinstance(arg[0], bool)
+                    and isinstance(arg[1], FormatValue)
+                ):
+                    raise TypeError(
+                        "operation %s argument must be a tuple[bool, FormatValue], "
+                        "got %s (value=%s)" % (name, type(arg).__name__, str(arg))
+                    )
+            elif not isinstance(arg, FormatValue):
+                if isinstance(arg, int):
+                    try:
+                        arg = FormatValue(arg)
+                    except Exception as e:
+                        raise TypeError(
+                            "operation %s argument must be a FormatValue] "
+                            "got %s (value=%s)" % (name, type(arg).__name__, str(arg))
+                        ) from e
+                else:
+                    raise TypeError(
+                        "operation %s argument must be a FormatValue] "
+                        "got %s (value=%s)" % (name, type(arg).__name__, str(arg))
+                    )
+
         elif opcode_has_argument(opcode):
             _check_arg_int(arg, name)
